%===================================================   
% This codes conduct the varaince decompositions in frequency domain
% without sentiment shock but with measurement error: six shocks 
% replicates "No Sentiment" lines in Table 3
% This version: May 27, 2015   
%===================================================        
clear all
close all

% set calibrated parameters
alpha = .3;   % capital share in production 
beta  = .99;  % discounting factor
del   = .025; % depreciation rate
dele  = .020; % exit rate 
Growgr= .42;  % growth rate of output
Growgz= 1.21; % growth rate of investment-specific shock
IY    = .2;   % Investment-output ratio
K0K   = .2;   % K0/K, capital stock of new firm to total capital stock ratio
omg   = .5;    % fraction of new firms that with bubble
gamma = 0;    % indivisible labor
% expected growth rate of bubble at steady state:
% we calibrate it as Rf/(output growth), we use Rf as 3-month treasury
% bill rate averaged from 1975Q2-2010Q4 and adjusted by the expected inflation
% the number is 1.0017;
theta = 1.0017/(1+Growgr/100);  % theta is 0.9975 which is very close to 1.    

save Cali_Para...
alpha beta del dele theta Growgr Growgz omg gamma IY K0K;


load EstBubbleNothet_results

sig_lam_z=oo_.posterior_mode.shocks_std.e_lam_zt;
sig_lam_a=oo_.posterior_mode.shocks_std.e_lam_at;
sig_v_a  =oo_.posterior_mode.shocks_std.e_v_at;
sig_psi  =oo_.posterior_mode.shocks_std.e_psit;
sig_xi   =oo_.posterior_mode.shocks_std.e_xit;
sig_err   =oo_.posterior_mode.shocks_std.e_errt;

BigOmg   =oo_.posterior_mode.parameters.BigOmg;
h        =oo_.posterior_mode.parameters.h;
del2     =oo_.posterior_mode.parameters.del2;
mu       =oo_.posterior_mode.parameters.mu;


xi    =oo_.posterior_mode.parameters.xi;

rho_lam_z=oo_.posterior_mode.parameters.rho_lam_z;   % permanent z shock
rho_lam_a=oo_.posterior_mode.parameters.rho_lam_a;   % permanent a shock
rho_v_a  =oo_.posterior_mode.parameters.rho_v_a;   % transitory a shock
rho_psi  =oo_.posterior_mode.parameters.rho_psi;   % shock to labor supply
rho_xi   =oo_.posterior_mode.parameters.rho_xi;   % shock to collateral constraint

fc1    =oo_.posterior_mode.parameters.fc1;
fc2    =oo_.posterior_mode.parameters.fc2;
fc3    =oo_.posterior_mode.parameters.fc3;

save Est_Para BigOmg h del2 mu theta xi ...  
     rho_lam_z rho_lam_a rho_v_a rho_psi rho_xi...
     sig_lam_z sig_lam_a sig_v_a sig_psi sig_xi sig_err...
     fc1 fc2 fc3;
 

%========================Impulse Responses&Variance Decomposition===================================           
% variance decompositions for aggregate variable: 
% variables are corresponding to the nonstationary series excluding linear time trend
dynare BubbleImpNothet.mod 

DelFiles
rmdir('BubbleImpNothet','s')
delete BubbleImpNothet.m
delete BubbleImpNothet_results.mat
%==========================
% In Dynare:
% the policy function have the form:
% yt=ghx_dr*yt(-1)+ghu_dr*ut
% the order of variables are in DR-order
% the following code transform the Dynare solution form into state-space
% form:
% yt=Mcx*xt(-1)+Mcu*ut;
% xt=Msx*xt(-1)+Msu*ut;
% where yt is control variables, xt is the state variables
% ut is the shocks with var-cov matrix Sigma*Sigma'
%==========================

n_var=M_.orig_endo_nbr; % number of original total variables


n_all=M_.endo_nbr;      % number of total variables defined by Dynare


% load coef-matrix
ghx_dr=oo_.dr.ghx;  
ghu_dr=oo_.dr.ghu; 

n_states=size(ghx_dr,2); % num of states variables
n_static=M_.nstatic; % num of static variables

ghx_dr_norm=ghx_dr(oo_.dr.inv_order_var,:);
ghu_dr_norm=ghu_dr(oo_.dr.inv_order_var,:);


% the order is DLogC DLogI DLogSP DLogY DLogZP LogN FCt
Msx=ghx_dr(n_static+1:n_static+n_states,:);
Msu=ghu_dr(n_static+1:n_static+n_states,:);

Mcx=ghx_dr_norm(24:30,:);
Mcu=ghu_dr_norm(24:30,:);


sigvec=diag([sig_lam_z sig_lam_a sig_v_a sig_psi sig_xi sig_err]');

n_u=size(ghu_dr,2);
% the range of frequency is 6~32 quarters

quart_upp=6;
quart_low=32;

[wvec,coef] = qnwtrap(2000,2*pi/quart_upp,2*pi/quart_low);

% Compute spectral density

for jj=1:n_u
    
    % define the std for each shock
    Sigma =diag(sigvec(jj,:));
    
    for ii=1:length(wvec);
       
        % frequency at w
        w=wvec(ii);
        
        % compute the spectral density for linearized variables
        Sp=SpectrumDSGE(w,Msx,Msu,Mcx,Mcu,Sigma);
        
        % define the spectrum of 1/(1-L)
        Sp_Lag=real(1/(1-exp(-1i*w))/(1-exp(1i*w)));
        
        % compute the spectral density for level
        SpMat(ii,:,jj)=[Sp_Lag;Sp_Lag;Sp_Lag;Sp_Lag;Sp_Lag;1;1]'.*Sp;

        
    end
    
    % integral spctral density for particular range of frequency
    Inte_SpMat(:,jj) = coef'*SpMat(:,:,jj);
end

% compute the variance
Sp_all=sum(Inte_SpMat,2)*ones(1,6);

% compute the percentage of fluctuations explained by each shock
VD=Inte_SpMat./Sp_all*100;

% report results

VD_SP=VD(3,:);
VD_C =VD(1,:);
VD_I =VD(2,:);
VD_Y =VD(4,:);
VD_L =VD(6,:);
% report results

disp('-------------------Variance decomposition-----------------------------------------------------------------')
disp(' Zgrowth     Agrowth     Atransitory      Labor       Collateral  MeaErr ');
disp('                        Stock Price')
disp( num2str(VD_SP));
disp('                        ')
disp('                        Consumption ')
disp( num2str(VD_C ));
disp('                        ')
disp('                        Investment ')
disp( num2str(VD_I ));
disp('                        ')
disp('                        Output ')
disp( num2str(VD_Y ));
disp('                        ')
disp('                        Labor ')
disp( num2str(VD_L ));
disp('                        ')