%===================================================   
% This is the main code to do historial decomposition in year-on-year growth rate
% for the benchmark model, replicates Figure 4
% (i)   IT bubble episode
% (ii)  Financial crisis episode
% This version: May 27, 2015
%===================================================        
clear all
close all;

% set calibrated parameters
alpha = .30;   % capital share in production 
beta  = .99;  % discounting factor
del   = .025; % depreciation rate
dele  = .020; % exit rate 
Growgr= .42;  % growth rate of output
Growgz= 1.21; % growth rate of investment-specific shock
IY    = .2;   % Investment-output ratio
K0K   = .2;   % K0/K, capital stock of new firm to total capital stock ratio
omg   =.5;    % fraction of new firms that with bubble
gamma = 0;    % indivisible labor

% expected growth rate of bubble at steady state:
% we calibrate it as Rf/(output growth), we use Rf as 3-month treasury
% bill rate averaged from 1975Q2-2010Q4 and adjusted by the expected inflation
% the number is 1.0017;
theta = .9975;   

save Cali_Para...
alpha beta del dele theta Growgr Growgz omg gamma IY K0K;

load EstBubbleBench_results
% set parameter value with posterior mode
% std for six shocks
sig_lam_z=oo_.posterior_mode.shocks_std.e_lam_zt;  % permanent z shock
sig_lam_a=oo_.posterior_mode.shocks_std.e_lam_at;  % permanent a shock
sig_v_a  =oo_.posterior_mode.shocks_std.e_v_at;    % transitory a shock
sig_the  =oo_.posterior_mode.shocks_std.e_thet;    % shock to theta
sig_psi  =oo_.posterior_mode.shocks_std.e_psit;    % shock to labor supply
sig_xi   =oo_.posterior_mode.shocks_std.e_xit;      % shock to collateral constraint

BigOmg   =oo_.posterior_mode.parameters.BigOmg; % investment adjustment cost
h        =oo_.posterior_mode.parameters.h;      % habit formation
del2     =oo_.posterior_mode.parameters.del2;   % elasticity of cdf at cut-off: Phi'/(1-Phi)*(epslon*)
mu       =oo_.posterior_mode.parameters.mu;     % capacity utilization parameter, del''/del'
xi       =oo_.posterior_mode.parameters.xi;     % financial constraint coefficient

% AR(1) coefficient for six shocks
rho_lam_z=oo_.posterior_mode.parameters.rho_lam_z;   % permanent z shock
rho_lam_a=oo_.posterior_mode.parameters.rho_lam_a;   % permanent a shock
rho_v_a  =oo_.posterior_mode.parameters.rho_v_a;   % transitory a shock
rho_the  =oo_.posterior_mode.parameters.rho_the;   % shock to theta
rho_psi  =oo_.posterior_mode.parameters.rho_psi;   % shock to labor supply
rho_xi   =oo_.posterior_mode.parameters.rho_xi;    % shock to collateral constraint

% parameters for the observation equation of financial condition index
fc1      =oo_.posterior_mode.parameters.fc1;
fc2      =oo_.posterior_mode.parameters.fc2;
fc3      =oo_.posterior_mode.parameters.fc3;

save Est_Para BigOmg h del2 mu xi fc1 fc2 fc3...  
     rho_lam_z rho_lam_a rho_v_a rho_the rho_psi rho_xi...
     sig_lam_z sig_lam_a sig_v_a sig_the sig_psi sig_xi;

% to get smoothed variables at posterior mean 
dynare BenchmarkSmoother

DelFiles
delete BenchmarkSmoother.m
rmdir  ('BenchmarkSmoother','s')

%---------------------------------------  
% smoothed shock series 
ut=[oo_.SmoothedShocks.e_lam_zt'
    oo_.SmoothedShocks.e_lam_at'
    oo_.SmoothedShocks.e_v_at'
    oo_.SmoothedShocks.e_thet'
    oo_.SmoothedShocks.e_psit'
    oo_.SmoothedShocks.e_xit'];

YYt=[...
oo_.SmoothedVariables.Nt' 
oo_.SmoothedVariables.Xt' 
oo_.SmoothedVariables.DLogC' 
oo_.SmoothedVariables.DLogI' 
oo_.SmoothedVariables.LogN' 
oo_.SmoothedVariables.DLogSP' 
oo_.SmoothedVariables.DLogY' 
oo_.SmoothedVariables.DLogZP' 
oo_.SmoothedVariables.FCt' 
oo_.SmoothedVariables.SPt' 
oo_.SmoothedVariables.Pt' 
oo_.SmoothedVariables.Yt' 
oo_.SmoothedVariables.Kt' 
oo_.SmoothedVariables.lam_zt' 
oo_.SmoothedVariables.lam_at' 
oo_.SmoothedVariables.v_at' 
oo_.SmoothedVariables.psit' 
oo_.SmoothedVariables.thet' 
oo_.SmoothedVariables.Ct' 
oo_.SmoothedVariables.It' 
oo_.SmoothedVariables.mt' 
oo_.SmoothedVariables.xit' 
oo_.SmoothedVariables.Bt' 
oo_.SmoothedVariables.Qt' 
oo_.SmoothedVariables.ut' 
oo_.SmoothedVariables.et' 
oo_.SmoothedVariables.grt' 
oo_.SmoothedVariables.gzt' 
oo_.SmoothedVariables.Lamt'];

%=========================Historical Decomposition=====================================


% Now we decompose the series into different exogeneous shocks

n_obs=size(ut,2); % number of observation in data

T=1975.25:.25:2010.75;

id_vec_mat(:,1)=[01;01;01;01;01;01];
id_vec_mat(:,2)=[01;01;01;00;01;01]; % turn off the sentiment shock 

% 1/0 means the shock is turned on/off
% the first shock:  zgrowth/ baised technology growth shock
% the second shock: agrwoth/ TFP growth
% the third shock: atrasitory
% the fourth: sentiments shocks
% the five shock: labor supply shocks
% the last shock: financial shocks.

LoadDataSentiFin_75
%---------------------------------------------------------------------
% Historical decomposition for IT bubble episode
idx_beg=find(T==1995-1); % starts from 1995
idx_end=find(T==2004);
n_period=idx_end-idx_beg+1;
gr   = Growgr/100+1; % 1+growth rate of output
   % LogY =cumsum(DLogY(idx_beg:idx_end));
    LogC =cumsum(DLogC(idx_beg:idx_end));
    LogI =cumsum(DLogI(idx_beg:idx_end));
    LogSP=cumsum(DLogSP(idx_beg:idx_end));
    LogN =LogN(idx_beg:idx_end);
    LogY =log(exp(LogC)+exp(LogI));
% compute the year-on-year growth rate in data

for j=5:n_period
    DLogY_yoy(j-4,1)=LogY(j)-LogY(j-4);
    DLogC_yoy(j-4,1)=LogC(j)-LogC(j-4);
    DLogI_yoy(j-4,1)=LogI(j)-LogI(j-4);
    DLogSP_yoy(j-4,1)=LogSP(j)-LogSP(j-4);
    DLogN_yoy(j-4,1)=LogN(j)-LogN(j-4);
end

% compute the year-on-year growth rate in meanl for different shocks

for i=1:2
    
    id_vec=id_vec_mat(:,i);
    [DLogCt_shock,DLogIt_shock,LogNt_shock,DLogSPt_shock,DLogYt_shock]=ShockHistDecomp(ut,YYt,n_obs,gr,id_vec,idx_beg,idx_end);  
    %LogYt =cumsum([LogY(1) ;DLogYt_shock]);
    LogCt =cumsum([LogC(1) ;DLogCt_shock]);
    LogIt =cumsum([LogI(1) ;DLogIt_shock]);
    LogSPt=cumsum([LogSP(1);DLogSPt_shock]);
    LogNt =[LogN(1);LogNt_shock];
    LogYt =log(exp(LogCt)+exp(LogIt));
    
    for j=5:n_period
        DLogYt_yoy(j-4) =LogYt(j)-LogYt(j-4);
        DLogCt_yoy(j-4) =LogCt(j)-LogCt(j-4);
        DLogIt_yoy(j-4) =LogIt(j)-LogIt(j-4);
        DLogSPt_yoy(j-4)=LogSPt(j)-LogSPt(j-4);
        DLogNt_yoy(j-4) =LogNt(j)-LogNt(j-4);
    end
    DLogYt_Shock_mat(:,i) =DLogYt_yoy;
    DLogCt_Shock_mat(:,i) =DLogCt_yoy;
    DLogIt_Shock_mat(:,i) =DLogIt_yoy;
    DLogNt_Shock_mat(:,i) =DLogNt_yoy;
    DLogSPt_Shock_mat(:,i)=DLogSPt_yoy;
end

nn=size(DLogSPt_Shock_mat,1);
    
figure(1) 
subplot(2,2,1)

barH=max([DLogSP_yoy;DLogSPt_Shock_mat(:,2)]);
barH=(1+0.4*sign(barH))*barH;
ymin=min([DLogSP_yoy;DLogSPt_Shock_mat(:,2)]);
ymin=(1-0.4*sign(ymin))*ymin;
fill([2001 2001 2001.75 2001.75],[ymin barH barH ymin],[.8,.8,.8]); hold on
ylim([ymin,barH])
xlim([T(idx_beg) T(idx_end)])

h1=plot(T(idx_beg+4:idx_end),DLogSP_yoy,'LineWidth',2);hold on;
h2=plot(T(idx_beg+4:idx_end),[DLogSP_yoy(1);DLogSPt_Shock_mat(2:end,1)],'-*','LineWidth',2);hold on
h3=plot(T(idx_beg+4:idx_end),[DLogSP_yoy(1);DLogSPt_Shock_mat(2:end,2)],'r--','LineWidth',2);


title('Stock price')
grid on

subplot(2,2,2)
                    
barH=max([DLogY_yoy;DLogYt_Shock_mat(:,2)]);
barH=(1+0.4*sign(barH))*barH;
ymin=min([DLogY_yoy;DLogYt_Shock_mat(:,2)]);
ymin=(1-0.4*sign(ymin))*ymin;
fill([2001 2001 2001.75 2001.75],[ymin barH barH ymin],[.8,.8,.8]); hold on
ylim([ymin,barH])
xlim([T(idx_beg) T(idx_end)])

h1=plot(T(idx_beg+4:idx_end),DLogY_yoy,'LineWidth',2);hold on;
h2=plot(T(idx_beg+4:idx_end),[DLogY_yoy(1);DLogYt_Shock_mat(2:nn,1)],'-*','LineWidth',2);hold on
h3=plot(T(idx_beg+4:idx_end),[DLogY_yoy(1);DLogYt_Shock_mat(2:nn,2)],'r--','LineWidth',2);
grid on
title('Output')


%print -depsc2 Figure4.eps

%---------------------------------------------------------------------
% Historical decomposition for financial crisis episode
idx_beg=find(T==2004-1);  % starts from 2004
idx_end=find(T==2010.75);
n_period=idx_end-idx_beg+1;

LoadDataSentiFin_75

    LogC =cumsum(DLogC(idx_beg:idx_end));
    LogI =cumsum(DLogI(idx_beg:idx_end));
    LogSP=cumsum(DLogSP(idx_beg:idx_end));
    LogY =log(exp(LogC)+exp(LogI));
    LogN =LogN(idx_beg:idx_end);
    
% compute the year-on-year growth rate in data
clear DLogC_yoy DLogI_yoy DLogSP_yoy DLogY_yoy DLogN_yoy
for j=5:n_period
    DLogY_yoy(j-4,1)=LogY(j)-LogY(j-4);
    DLogC_yoy(j-4,1)=LogC(j)-LogC(j-4);
    DLogI_yoy(j-4,1)=LogI(j)-LogI(j-4);
    DLogSP_yoy(j-4,1)=LogSP(j)-LogSP(j-4);
    DLogY_yoy(j-4,1)=LogY(j)-LogY(j-4);
    DLogN_yoy(j-4,1)=LogN(j)-LogN(j-4);
end

% compute the year-on-year growth rate in meanl for different shocks
clear DLogCt_yoy DLogIt_yoy DLogSPt_yoy DLogYt_yoy DLogNt_yoy...
      DLogYt_Shock_mat DLogCt_Shock_mat DLogIt_Shock_mat DLogSPt_Shock_mat DLogNt_Shock_mat
for i=1:2
    
    id_vec=id_vec_mat(:,i);
    [DLogCt_shock,DLogIt_shock,LogNt_shock,DLogSPt_shock,DLogYt_shock]=ShockHistDecomp(ut,YYt,n_obs,gr,id_vec,idx_beg,idx_end);  
    
    LogCt =cumsum([LogC(1) ;DLogCt_shock]);
    LogIt =cumsum([LogI(1) ;DLogIt_shock]);
    LogSPt=cumsum([LogSP(1);DLogSPt_shock]);
    LogNt =[LogN(1);LogNt_shock];
    LogYt =log(exp(LogCt)+exp(LogIt));


    
    for j=5:n_period
        DLogYt_yoy(j-4)=LogYt(j)-LogYt(j-4);
        DLogCt_yoy(j-4)=LogCt(j)-LogCt(j-4);
        DLogIt_yoy(j-4)=LogIt(j)-LogIt(j-4);
        DLogSPt_yoy(j-4)=LogSPt(j)-LogSPt(j-4);
        DLogYt_yoy(j-4)=LogYt(j)-LogYt(j-4);
        DLogNt_yoy(j-4)=LogNt(j)-LogNt(j-4);
    end
    DLogYt_Shock_mat(:,i)=DLogYt_yoy;
    DLogCt_Shock_mat(:,i)=DLogCt_yoy;
    DLogIt_Shock_mat(:,i)=DLogIt_yoy;
    DLogNt_Shock_mat(:,i)=DLogNt_yoy;
    DLogSPt_Shock_mat(:,i)=DLogSPt_yoy;
end
nn=size(DLogSPt_Shock_mat,1);

figure(1) 
subplot(2,2,3)

barH=max([DLogSP_yoy;DLogSPt_Shock_mat(:,2)]);
barH=(1+0.4*sign(barH))*barH;
ymin=min([DLogSP_yoy;DLogSPt_Shock_mat(:,2)]);
ymin=(1-0.4*sign(ymin))*ymin;
fill([2007.75 2007.75 2009.25  2009.25],[ymin barH barH ymin],[.8,.8,.8]); hold on
ylim([ymin,barH])
xlim([T(idx_beg) T(idx_end)])

h1=plot(T(idx_beg+4:idx_end),DLogSP_yoy,'LineWidth',2);hold on;
h2=plot(T(idx_beg+4:idx_end),[DLogSP_yoy(1);DLogSPt_Shock_mat(2:end,1)],'-*','LineWidth',2);hold on
h3=plot(T(idx_beg+4:idx_end),[DLogSP_yoy(1);DLogSPt_Shock_mat(2:end,2)],'r--','LineWidth',2);

h=[h1 h2 h3];

title('Stock price')
grid on

subplot(2,2,4)
                    
barH=max([DLogY_yoy;DLogYt_Shock_mat(:,2)]);
barH=(1+0.4*sign(barH))*barH;
ymin=min([DLogY_yoy;DLogYt_Shock_mat(:,2)]);
ymin=(1-0.4*sign(ymin))*ymin;
fill([2007.75 2007.75 2009.25  2009.25],[ymin barH barH ymin],[.8,.8,.8]); hold on
ylim([ymin,barH])
xlim([T(idx_beg) T(idx_end)])

h1=plot(T(idx_beg+4:idx_end),DLogY_yoy,'LineWidth',2);hold on;
h2=plot(T(idx_beg+4:idx_end),[DLogY_yoy(1);DLogYt_Shock_mat(2:nn,1)],'-*','LineWidth',2);hold on
h3=plot(T(idx_beg+4:idx_end),[DLogY_yoy(1);DLogYt_Shock_mat(2:nn,2)],'r--','LineWidth',2);
grid on
title('Output')
h=[h1 h2 h3];

legend(h,'Data','model','Sentiment', 'Location','SouthWest');


%print -depsc2 Recessions.eps
